import json
from termcolor import colored, cprint
from prettytable import PrettyTable

class JsonFormatter(object):
    """JsonFormatter formats the program details in Json"""

    def format(self, items, show_commands):
        """Formats the items into Json

        Args:
            items (list): The results of the program assessment
            show_commands (boolean): Indicates if the commands should be shown for debugging
        """  

        jdict = {item.name: item.current for item in items}
        return  json.dumps(jdict, indent=4, sort_keys=True)
        

class TableFormatter(object):
    """TableFormatter formats the program details to a table"""

    def format(self, items, show_commands):
        """Formats the items into a table

        Args:
            items (list): The results of the program assessment
            show_commands (boolean): Indicates if the commands should be shown for debugging
        """  

        items.sort(key=lambda item: item.name)

        table = PrettyTable(self.get_headers(show_commands))
        for item in items:
            # current = colored(item.current, 'magenta')
            table.add_row(item.get_values(show_commands))

        return table


    def get_headers(self, show_commands):
        """Returns the headers to be used for the table

        Args:
            show_commands (boolean): Indicates if the commands should be includes
        """  

        if show_commands:
            return ['Name', 'Current Command', 'Current', 'Latest Command', 'Latest']
        else:
            return ['Name', 'Current', 'Latest']



import click
import os
import csv
import concurrent.futures
from termcolor import colored, cprint
from env.config import Config
from env.programs import Programs
from env.formatters import TableFormatter, JsonFormatter
from env.outputters import ConsoleOutputter, FileOutputter

@click.group()
def environment():
    """Defines the group for environment commands"""
    pass


@environment.command()
@click.option('--out', '-o', help='The output file for the results')
@click.option('--show-commands', '-sc', is_flag=True, default=False)
def list(out, show_commands):
    """The command for displaying the current environment

    Args:
        out (str): Optional file name to output the 
        show_commands (boolean): Indicates in the commands should be shown in the output
    """
    
    command = Programs(TableFormatter(), ConsoleOutputter())

    path = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(path, "./programs.csv")

    with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
        with open(path) as csv_file:
            csv_reader = csv.reader(csv_file, delimiter = ',')
            
            for row in csv_reader:
                executor.submit(command.add_version, row)

    command.output(show_commands)


import json

class ConsoleOutputter(object):
    """ConsoleOutputter writes the result to the console"""

    def output(self, result):
        """Prints to the console
        
        Args:
            result (object): The result of the assessment to be output
        """  

        print()
        print(result)
        print()


class FileOutputter(object):
    """FileOutputter writes the result to a specified file"""

    def __init__(self, file_name):
        """Initializes the FileOutputter

        Args:
            file_name (str): Name of the file to output
        """

        self.file_name = file_name
        
    def output(self, result):
        """Outputs the results to a file
        
        Args:
            result (object): The result of the assessment to be output
        """

        with open(self.file_name, 'w') as file:
            json.dump(result, file)
    




import json

class Config(object):
    """Represents the config for one of the programs"""

    def __init__(self, data):
        """Initializes the config based on a row of data
        
        Args:
            data (array): An data of the data that provides inputs
        """
        
        self.name = data[0]
        self.cl_name = data[1]
        self.brew_name = data[2]
        self.type = data[3]
        self.version_switch = data[4]
        self.current_formatter = data[5]
        self.position = int(data[6])

        self.current_command = "{} {}".format(self.cl_name, self.version_switch).split()
        self.latest_command = "{} outdated {}".format(self.type, self.brew_name).split()
        self.update_command = "{} upgrade {}".format(self.type, self.brew_name).split()



class Program(object):
    """Represents the results of a program result"""
    
    def __init__(self, config, current, latest):
        """Initializes Program results
        
        Args:
            config (Config): The config used to retrieve the results
            current (str): The current version installed locally
            latest (str): The latest version available
        """ 

        self.config = config
        self.name = str(config.name)
        self.current = self.format_version(current)
        self.latest = self.format_version(latest)


    def format_version(self, version):
        """Formats a version number
        
        Args:
            version (str): The raw version number before formatting
        """ 

        if version is not None and version is not "" and version[:1] is not "v":
            version = "v{}".format(version)
        
        return version


    def get_values(self, show_commands):
        """Retrieves the values for the program
        
        Args:
            show_commands (boolean): Determines if the commands should be shown
        """ 

        if show_commands:
            return [self.name, self.config.current_command, 
                self.current, self.config.latest_command, self.latest]
        else:
            return [self.name, self.current, self.latest]


#!/usr/bin/env python

import click
from env.commands import environment

@click.group()
def cli():
    """Entry point for the cli"""
    pass

cli.add_command(environment)

if __name__ == '__main__':
    """Enables CLI to be run directly"""
    cli()




import threading
from env.config import Config
from env.program import Program
from subprocess import Popen, DEVNULL, PIPE

class Programs(object):
    """Implements the List command for the environment"""
    
    def __init__(self, formatter, outputter):
        """Initializes the environment wrapper

        Args:
            formatter (Formatter): The formatter for program details
            outputter (Outputter): The outputter for program details
        """

        self.formatter = formatter
        self.outputter = outputter
        self.items = []

    def execute_command(self, config, command, formatter = None):
        
        p = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        raw_result, err = p.communicate(b"input data that is passed to subprocess' stdin")
        rc = p.returncode

        if rc is 0:

            formatted_result = raw_result.rstrip().decode("utf-8") 

            if formatter is None:
                return formatted_result
            else:
                return getattr(self, formatter)(config, formatted_result)
            
        else:
            pass


    def add_version(self, row):
        """Runs the program to retrieve the program version

        Args:
            args (list): The program name and required args to return the version info.
            formatter (Formatter): The outputter of the program details.
        """

        config = Config(row)

        try:
            
            current = self.execute_command(config, config.current_command, config.current_formatter)
            latest = self.execute_command(config, config.latest_command)

            program = Program(config, current, latest)
            self.items.append(program)
            
        except FileNotFoundError as error:
            program = Program(config)
            self.items.append(program)


    def extract(self, config, output):
        """ """

        items = output.split()
        return items[config.position]

    def extract_go(self, config, output):
        """ """

        items = output.split()
        return items[2][2:]

    def extract_docker(self, config, output):
        """ """

        items = output.split()
        return items[2][:-1]


    def output(self, show_commands):
        """Outputs to based on the provides args"""
        
        # format(self, items, show_commands):
        # def output(self, result):

        result = self.formatter.format(self.items, show_commands)
        self.outputter.output(result)